#include <qpixmap.h>
#include <qbitmap.h>
#include <qaccel.h>
#include <qkeycode.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "qplaycd.moc"
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "cdaudio.xbm"
#include "cdeject.xbm"
#include "cdffwd.xbm"
#include "cdicnfy.xbm"
#include "cdnext.xbm"
#include "cdpause.xbm"
#include "cdplay.xbm"
#include "cdpower.xbm"
#include "cdprev.xbm"
#include "cdrewd.xbm"
#include "cdstop.xbm"
#include "cdthis.xbm"

Atom _XA_GNUSTEP_WM_FUNC;

Display *dpy;

Window leader;
XWMHints *hints;
#define DEFAULTDEVICE "/dev/cdrom"

char txtdev[256]=DEFAULTDEVICE;

void scanArgs(int argc, char **argv);

int main(int argc, char **argv){

   scanArgs(argc, argv);

   QApplication app(argc,argv);
   app.setStyle(WindowsStyle);
   mapp = &app;
   MSignals msig;

   // create a Palette for later use
   QColor mcol("red");
   QColorGroup mcg(mcol,mcol.dark(),mcol.dark(),mcol,mcol,mcol,white);
   QColorGroup m0cg(mcol,mcol.dark(),black,black,black,mcol.dark(175),white);
   QColorGroup m1cg(mcol,mcol.dark(),black,black,black,mcol,white);
   QPalette mpal(mcg,mcg,mcg);
   QPalette m0pal(m0cg,m0cg,m0cg);
   QPalette m1pal(m1cg,m1cg,m1cg);
   p0pal=&m0pal;
   p1pal=&m1pal;

   // create Main Window
   QWidget mwidget(0);
   mwidget.setFixedSize(430,60);
   mwidget.setCaption("QPlayCD");

   // create Frame for displays
   QFrame mframe1(&mwidget);
   mframe1.setGeometry(0,0,500,60);
   mframe1.setFrameStyle(QFrame::WinPanel | QFrame::Raised);

   // create Track / Status display
   QLabel mlabel1(&mwidget);
   mlabel1.setFont(QFont("helvetica",14,QFont::DemiBold));
   mlabel1.setGeometry(2,2,119,15);
   mlabel1.setMargin(3);
   mlabel1.setPalette(mpal);
   pmatrix1=&mlabel1;

   // create Time Display LED matrix
   QLCDNumber mtime2(&mwidget);
   mtime2.setGeometry(2,17,119,40);
   mtime2.setFrameStyle(QFrame::NoFrame);
   mtime2.setPalette(mpal);
   mtime2.setNumDigits(5);
   mtime2.setSegmentStyle(QLCDNumber::Filled);
   mtime2.display("00:00");
   ptime2=&mtime2;

   // create CD Digital Audio icon
   QLabel label(&mwidget);
   label.setPalette(mpal);
   label.setGeometry(121,2,56,55);
   label.setAlignment(AlignHCenter | AlignBottom);
   label.setPixmap(QBitmap(cdaudio_width,cdaudio_height,(const unsigned char *)cdaudio_bits,TRUE));

   // create CD Status icons
   for(int i=0;i<4;i++){
      sicon[i]=new QLabel(&mwidget);
      sicon[i]->setPalette(m0pal);
      sicon[i]->setGeometry(121+14*i,2,14,14);
      sicon[i]->setAlignment(AlignCenter);
   }

   // create CD Control buttons
   QButtonGroup mbg(&mwidget);
   mbg.setGeometry(185,5,245,50);
   mbg.setFrameStyle(QFrame::NoFrame);
   QFrame frame2(&mbg);
   frame2.setGeometry(0,0,250,20);
   frame2.setFrameStyle(QFrame::Panel | QFrame::Sunken);
   QFrame frame3(&mbg);
   frame3.setGeometry(0,30,250,20);
   frame3.setFrameStyle(QFrame::Panel | QFrame::Sunken);
   QObject::connect(&mbg,SIGNAL(clicked(int)),&msig,SLOT(btnGroupClicked(int)));
   QPushButton *mpb;

   (mpb = new QPushButton("STOP",&mbg))->setGeometry(3,3,35,14);
   mpb->setPixmap(QBitmap(cdstop_width,cdstop_height,(const unsigned char *)cdstop_bits,TRUE));
   sicon[0]->setPixmap(QBitmap(cdstop_width,cdstop_height,(const unsigned char *)cdstop_bits,TRUE));
   ppb[0]=mpb;

   (mpb = new QPushButton("PAUSE",&mbg))->setGeometry(38,3,35,14);
   mpb->setPixmap(QBitmap(cdpause_width,cdpause_height,(const unsigned char *)cdpause_bits,TRUE));
   sicon[1]->setPixmap(QBitmap(cdpause_width,cdpause_height,(const unsigned char *)cdpause_bits,TRUE));
   ppb[1]=mpb;

   (mpb = new QPushButton("PLAY",&mbg))->setGeometry(73,3,35,14);
   mpb->setPixmap(QBitmap(cdplay_width,cdplay_height,(const unsigned char *)cdplay_bits,TRUE));
   sicon[2]->setPixmap(QBitmap(cdplay_width,cdplay_height,(const unsigned char *)cdplay_bits,TRUE));
   ppb[2]=mpb;

   (mpb = new QPushButton("PREV",&mbg))->setGeometry(3,33,35,14);
   mpb->setPixmap(QBitmap(cdprev_width,cdprev_height,(const unsigned char *)cdprev_bits,TRUE));
   ppb[3]=mpb;

   (mpb = new QPushButton("THIS",&mbg))->setGeometry(38,33,35,14);
   mpb->setPixmap(QBitmap(cdthis_width,cdthis_height,(const unsigned char *)cdthis_bits,TRUE));
   ppb[4]=mpb;

   (mpb = new QPushButton("NEXT",&mbg))->setGeometry(73,33,35,14);
   mpb->setPixmap(QBitmap(cdnext_width,cdnext_height,(const unsigned char *)cdnext_bits,TRUE));
   ppb[5]=mpb;

   (mpb = new QPushButton("REWD",&mbg))->setGeometry(128,3,35,14);
   mpb->setPixmap(QBitmap(cdrewd_width,cdrewd_height,(const unsigned char *)cdrewd_bits,TRUE));
   ppb[6]=mpb;

   (mpb = new QPushButton("EJECT",&mbg))->setGeometry(163,3,35,14);
   mpb->setPixmap(QBitmap(cdeject_width,cdeject_height,(const unsigned char *)cdeject_bits,TRUE));
   sicon[3]->setPixmap(QBitmap(cdeject_width,cdeject_height,(const unsigned char *)cdeject_bits,TRUE));
   ppb[7]=mpb;

   (mpb = new QPushButton("FFWD",&mbg))->setGeometry(198,3,35,14);
   mpb->setPixmap(QBitmap(cdffwd_width,cdffwd_height,(const unsigned char *)cdffwd_bits,TRUE));
   ppb[8]=mpb;

   (mpb = new QPushButton("ICNFY",&mbg))->setGeometry(198,33,35,14);
   mpb->setPixmap(QBitmap(cdicnfy_width,cdicnfy_height,(const unsigned char *)cdicnfy_bits,TRUE));
   ppb[9]=mpb;

   (mpb = new QPushButton("POWER",&mbg))->setGeometry(128,33,70,14);
   mpb->setPixmap(QBitmap(cdpower_width,cdpower_height,(const unsigned char *)cdpower_bits,TRUE));
   ppb[10]=mpb;

   // Create Keyboard Shortcuts
   QAccel macc(&mwidget);
   macc.insertItem(Key_7,0);
   macc.insertItem(Key_8,1);
   macc.insertItem(Key_9,2);
   macc.insertItem(Key_4,3);
   macc.insertItem(Key_5,4);
   macc.insertItem(Key_6,5);
   macc.insertItem(Key_1,6);
   macc.insertItem(Key_2,7);
   macc.insertItem(Key_3,8);
   macc.insertItem(Key_Period,9);
   macc.insertItem(Key_0,10);
   QObject::connect(&macc,SIGNAL(activated(int)),&msig,SLOT(btnGroupClicked(int)));

   // James begged me to put this in for WindowMaker
   XClassHint classHint;
   dpy=mwidget.x11Display();
   _XA_GNUSTEP_WM_FUNC = XInternAtom(dpy,"_GNUSTEP_WM_FUNCTION",FALSE);
   leader = mwidget.handle();
   classHint.res_name = "qplaycd";
   classHint.res_class="QPlayCD";
   XSetClassHint(dpy,leader,&classHint);
   hints=XAllocWMHints();
   hints->window_group=leader;
   hints->flags=WindowGroupHint;
   XSetWMHints(dpy,leader,hints);
   XSetIconName(dpy,leader,"QPlayCD");
   // James stopped begging here

   // create CD Control
   cdctl=new CDCtl(txtdev);

   // start Application
   app.setMainWidget(&mwidget);
   mwidget.show();
   int retval = app.exec();

   for(int i=0;i<4;i++)
      delete sicon[i];
   delete cdctl;

   return retval;
}

void scanArgs(int argc, char **argv){
   for(int i=1;i<argc;i++){
      if(strcmp(argv[i],"-h")==0 || strcmp(argv[i],"-help")==0 || strcmp(argv[i],"--help")==0){
         fprintf(stderr,"qplaycd - the Qt cd player\nrelease 0.2\n07/02/98\n\n");
         fprintf(stderr,"by Sam Hawker\nshawkie@geocities.com\n");
         fprintf(stderr,"http://www.geocities.com/SiliconValley/Vista/2471/\n\n");
         fprintf(stderr,"usage:\n\n   %s [options]\n\noptions:\n\n",argv[0]);
         fprintf(stderr,"   -h | -help | --help    display this help screen\n");
         fprintf(stderr,"   -d cd_device           use specified device  (rather than /dev/cdrom)\n");
         exit(0);
      }
      if(strcmp(argv[i],"-d")==0){
	 if(i<argc-1){
            i++;
            sprintf(txtdev,"%s",argv[i]);
         }
         continue;
      }
   }
}
